describe McDump::Memcached::Item do

  let(:items_id)        { "8" }
  let(:cache_key)       { SecureRandom.uuid }
  let(:expiration_time) { "1447214865" }
  let(:size_in_bytes)   { 321 }

  let(:args) do
    {
      items_id:        items_id,
      cache_key:       cache_key,
      expiration_time: expiration_time,
      size_in_bytes:   size_in_bytes
    }
  end

  let(:item) { described_class.new(args) }

  describe "::parse" do

    subject { described_class.parse(dump, items_id) }

    context "when the dump contains many lines" do

      let(:dump) do
        <<-DUMP
ITEM namespace:fc7c3121-30c9-478c-9662-99f8b9df8931 [407 b; 1447214865 s]
ITEM namespace:e81bbb97-d679-4cea-80c2-787950256083 [392 b; 1447214866 s]
ITEM namespace:f5ed8536-1265-486a-afbf-73a26828a0ce [413 b; 1447214867 s]
        DUMP
      end

      it "creates an item for each line containing the line cache keys" do
        expected_cache_keys = %w{
          namespace:fc7c3121-30c9-478c-9662-99f8b9df8931
          namespace:e81bbb97-d679-4cea-80c2-787950256083
          namespace:f5ed8536-1265-486a-afbf-73a26828a0ce
        }
        expected_cache_keys.each do |expected_cache_key|
          expect(described_class).to receive(:new).with(hash_including(cache_key: expected_cache_key))
        end

        subject
      end

      it "creates an item for each line containing the line expiration times converted to Time" do
        expected_expiration_times = %w{ 1447214865 1447214866 1447214867 }
        expected_expiration_times.each do |expected_expiration_time|
          expectation_as_time = Time.at(expected_expiration_time.to_i)
          expect(described_class).to receive(:new).with(hash_including(expiration_time: expectation_as_time))
        end

        subject
      end

      it "creates an item for each line containing the line sizes in bytes" do
        expected_sizes_in_bytes = [ 407, 392, 413 ]
        expected_sizes_in_bytes.each do |expected_size_in_bytes|
          expect(described_class).to receive(:new).with(hash_including(size_in_bytes: expected_size_in_bytes))
        end

        subject
      end

      it "creates an item for each line containing the provided items ID" do
        expect(described_class).to receive(:new).with(hash_including(items_id: items_id)).exactly(3).times

        subject
      end

      it "returns the created items" do
        items = (1..3).map { instance_double(described_class) }
        allow(described_class).to receive(:new).and_return(*items)

        expect(subject).to eql(items)
      end

    end

    context "when the dump is empty" do

      let(:dump) { "" }

      it "returns an empty array" do
        expect(subject).to eql([])
      end

    end

  end

  describe "#to_h" do

    subject { item.to_h }

    it "returns a hash containing the provided arguments" do
      expect(subject).to eql(args)
    end

    it "returns a copy of the provided arguments" do
      original_cache_key = cache_key

      hash = subject
      args[:cache_key] = "modified cache key"

      expect(hash[:cache_key]).to eql(original_cache_key)
    end

  end

end
