describe McDump::Telnet::Session do

  let(:connection_args) { { host: "some.host", port: 12345 } }
  let(:connection)      { instance_double(McDump::Telnet::Connection) }

  let(:session) { described_class.new(connection_args) }

  before(:example) { allow(McDump::Telnet::Connection).to receive(:new).and_return(connection) }

  describe "::open" do

    let(:open_block_result) { "some open block result"}
    let(:open_block)        { lambda { |session| open_block_result } }
    let(:session)           { instance_double(described_class, close: nil) }

    subject { described_class.open(connection_args, &open_block) }

    before(:example) { allow(described_class).to receive(:new).and_return(session) }

    it "creates a session with the provided connection settings" do
      expect(described_class).to receive(:new).with(connection_args)

      subject
    end

    it "invokes the provided block with the created session" do
      expect(open_block).to receive(:call).with(session)

      subject
    end

    it "closes the created session" do
      expect(session).to receive(:close)

      subject
    end

    it "returns the result of the block" do
      expect(subject).to eql(open_block_result)
    end

  end

  describe "constructor" do

    subject { session }

    it "opens a connection using the provided connection settings" do
      expect(McDump::Telnet::Connection).to receive(:new).with(connection_args)

      subject
    end

  end

  describe "#stats" do

    subject { session.stats }

    it "executes a command on the connection to retrieve the stats for items" do
      expect(connection).to receive(:execute).with("stats items")

      subject
    end

    it "returns the commands response" do
      response = (1..3).map { |i| "STAT line #{i}" }
      allow(connection).to receive(:execute).and_return(response)

      expect(subject).to eql(response)
    end

  end

  describe "#items" do

    let(:id)     { 12 }
    let(:number) { 987 }

    subject { session.items(id, number) }

    it "executes a command on the connection showing the number of items matching the id" do
      expect(connection).to receive(:execute).with("stats cachedump #{id} #{number}")

      subject
    end

    it "returns the commands response" do
      response = (1..3).map { |i| "ITEM line #{i}" }
      allow(connection).to receive(:execute).and_return(response)

      expect(subject).to eql(response)
    end

  end

  describe "#close" do

    subject { session.close }

    it "closes the telnet connection that was made when the session was created" do
      expect(connection).to receive(:close)

      subject
    end

  end

end
